from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

import pandas as pd
import urllib.request


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            """
            Parse council provided CSVs to get the latest bin collections for address
            """

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            # Create Selenium webdriver
            page = f"https://www.leeds.gov.uk/residents/bins-and-recycling/check-your-bin-day"

            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            wait = WebDriverWait(driver, 60)
            postcode_box = wait.until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_txtPostCode",
                    )
                )
            )
            postcode_box.send_keys(user_postcode)
            postcode_btn_present = wait.until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_btnSearchAddress",
                    )
                )
            )
            postcode_btn = wait.until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        '//*[@id="ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_btnSearchAddress"]',
                    )
                )
            )

            postcode_btn.send_keys(Keys.ENTER)

            dropdown_present = wait.until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//*[@id="ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_ddlAddressList"]/option',
                    )
                )
            )
            address_dropdown = wait.until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_ddlAddressList",
                    )
                )
            )

            dropdown_present.click()

            dropdownSelect = Select(address_dropdown)
            dropdownSelect.select_by_value(str(user_uprn))
            results = wait.until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "ctl00_ctl48_g_eea1a8ba_4306_488e_96f2_97f22038e29f_ctl00_BinResultsDetails",
                    )
                )
            )

            data = {"bins": []}  # dictionary for data
            soup = BeautifulSoup(driver.page_source, "html.parser")

            bin_types = soup.find_all("ul", class_="binCollectionTimesList")

            for bin_collection_dates in bin_types:
                bin_collection_list = bin_collection_dates.find_all("li", class_="")

                if bin_collection_list:
                    collection_dates = [
                        date.text.strip() for date in bin_collection_list
                    ]

                    # Convert the collection dates to the desired format
                    formatted_dates = [
                        datetime.strptime(date, "%A %d %b %Y").strftime(date_format)
                        for date in collection_dates
                    ]

                    # Extract the type of bin from the header
                    bin_type = bin_collection_dates.find_previous("h3").text.split()[0]

                    # Adding data to the 'bins' dictionary for each date
                    for date in formatted_dates:
                        dict_data = {"type": bin_type, "collectionDate": date}
                        data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            driver = create_webdriver(web_driver, headless)
            # driver = webdriver.Chrome(service=ChromeService(ChromeDriverManager().install()))
            driver.get(
                "https://webapps.sunderland.gov.uk/WEBAPPS/WSS/Sunderland_Portal/Forms/bindaychecker.aspx"
            )

            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "ContentPlaceHolder1_tbPostCode_controltext")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            inputElement_submit_button = WebDriverWait(driver, 30).until(
                EC.element_to_be_clickable((By.ID, "ContentPlaceHolder1_btnLLPG"))
            )
            inputElement_submit_button.click()

            addressList = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "ContentPlaceHolder1_ddlAddresses")
                )
            )
            selected_addressList = Select(addressList)
            for idx, addr_option in enumerate(selected_addressList.options):
                option_name = addr_option.accessible_name[0 : len(user_paon)]
                if option_name == user_paon:
                    break
            selected_addressList.select_by_index(idx)

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            household_bin_date = datetime.strptime(
                soup.find("span", {"id": "ContentPlaceHolder1_LabelHouse"}).get_text(
                    strip=True
                ),
                "%A %d %B %Y",
            )
            collections.append(("Household bin", household_bin_date))

            recycling_bin_date = datetime.strptime(
                soup.find("span", {"id": "ContentPlaceHolder1_LabelRecycle"}).get_text(
                    strip=True
                ),
                "%A %d %B %Y",
            )
            collections.append(("Recycling bin", recycling_bin_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            for item in ordered_data:
                dict_data = {
                    "type": item[0].capitalize(),
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise

        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()

        return data



import itertools

from bs4 import BeautifulSoup, Tag
from dateutil.parser import parse as date_parse

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_uprn = kwargs.get("uprn")

        api_url = (
            f"https://maps.westsuffolk.gov.uk/MyWestSuffolk.aspx?action=SetAddress&UniqueId={user_uprn}"
        )

        response = requests.get(api_url)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        def panel_search(cur_tag: Tag):
            """
            Helper function to find the correct tag
            """
            if cur_tag.name != "div":
                return False

            tag_class = cur_tag.attrs.get("class", None)
            if tag_class is None:
                return False

            parent_has_header = cur_tag.parent.find_all("h4", string="Bin collection days")
            if len(parent_has_header) < 1:
                return False

            return "atPanelData" in tag_class

        collection_tag = soup.body.find_all(panel_search)

        # Parse the resultant div
        for tag in collection_tag:
            text_list = list(tag.stripped_strings)
            # Create and parse the list as tuples of name:date
            for bin_name, collection_date in itertools.batched(text_list, 2):
                # Clean-up the bin_name
                bin_name_clean = bin_name.strip().replace("\r", "").replace("\n", "")
                bin_name_clean = re.sub(' +', ' ', bin_name_clean)

                # Parse the date
                next_collection = date_parse(collection_date)
                next_collection = next_collection.replace(year=datetime.now().year)

                dict_data = {
                    "type": bin_name_clean,
                    "collectionDate": next_collection.strftime(date_format),
                }

                data["bins"].append(dict_data)

        return data



import json
import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
import ssl
import urllib3


class CustomHttpAdapter(requests.adapters.HTTPAdapter):
    """Transport adapter" that allows us to use custom ssl_context."""

    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            block=block,
            ssl_context=self.ssl_context,
        )


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        headers = {
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Language": "en-GB,en;q=0.9",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "application/json; charset=utf-8",
            "Pragma": "no-cache",
            "Referer": "https://www.bathnes.gov.uk/webforms/waste/collectionday/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.5845.188 Safari/537.36",
            "X-Requested-With": "XMLHttpRequest",
        }

        session = requests.Session()
        ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
        ctx.options |= 0x4
        session.mount("https://", CustomHttpAdapter(ctx))

        requests.packages.urllib3.disable_warnings()
        response = session.get(
            f"https://www.bathnes.gov.uk/webapi/api/BinsAPI/v2/getbartecroute/{user_uprn}/true",
            headers=headers,
        )
        if response.text == "":
            raise ValueError(
                "Error parsing data. Please check the provided UPRN. "
                "If this error continues please open an issue on GitHub."
            )
        json_data = json.loads(response.text)

        data = {"bins": []}

        if len(json_data["residualNextDate"]) > 0:
            dict_data = {
                "type": "Black Rubbish Bin",
                "collectionDate": datetime.strptime(
                    json_data["residualNextDate"], "%Y-%m-%dT%H:%M:%S"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)
        if len(json_data["recyclingNextDate"]) > 0:
            dict_data = {
                "type": "Recycling Containers",
                "collectionDate": datetime.strptime(
                    json_data["recyclingNextDate"], "%Y-%m-%dT%H:%M:%S"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)
        if len(json_data["organicNextDate"]) > 0:
            dict_data = {
                "type": "Garden Waste",
                "collectionDate": datetime.strptime(
                    json_data["organicNextDate"], "%Y-%m-%dT%H:%M:%S"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



import logging
import traceback
from typing import Any, Generator, Callable

import pytest
from pytest_bdd import scenario, given, when, then, parsers
from functools import wraps

from step_helpers import file_handler
from uk_bin_collection.uk_bin_collection import collect_data

logging.basicConfig(level=logging.INFO, format="%(levelname)s - %(message)s")

@scenario("../features/validate_council_outputs.feature", "Validate Council Output")
def test_scenario_outline() -> None:
    pass

def handle_test_errors(func: Callable[..., Any]) -> Callable[..., Any]:
    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(f"Error in test '{func.__name__}': {e}")
            logging.error(traceback.format_exc())
            raise e
    return wrapper

@pytest.fixture
@handle_test_errors
def context() -> Generator[Any, None, None]:
    class Context:
        metadata: dict[str, Any]
        council: str
        parse_result: Any

    return Context()

@handle_test_errors
@given(parsers.parse("the council: {council_name}"))
def get_council_step(context: Any, council_name: str) -> None:
    council_input_data = file_handler.load_json_file("input.json")
    context.metadata = council_input_data[council_name]

@handle_test_errors
@when(parsers.parse("we scrape the data from {council}"))
def scrape_step(context: Any, council: str, headless_mode: str, local_browser: str, selenium_url: str) -> None:
    context.council = council

    args = [council, context.metadata["url"]]

    if "uprn" in context.metadata:
        uprn = context.metadata["uprn"]
        args.append(f"-u={uprn}")
    if "postcode" in context.metadata:
        postcode = context.metadata["postcode"]
        args.append(f"-p={postcode}")
    if "house_number" in context.metadata:
        house_number = context.metadata["house_number"]
        args.append(f"-n={house_number}")
    if "usrn" in context.metadata:
        usrn = context.metadata["usrn"]
        args.append(f"-us={usrn}")
    if headless_mode == "True":
        args.append("--headless")
    else:
        args.append("--not-headless")

    if local_browser == "False":
        args.append(f"-w={selenium_url}")
    if "skip_get_url" in context.metadata:
        args.append("-s")

    CollectData = collect_data.UKBinCollectionApp()
    CollectData.set_args(args)
    context.parse_result = CollectData.run()

@handle_test_errors
@then("the result is valid json")
def validate_json_step(context: Any) -> None:
    assert file_handler.validate_json(context.parse_result), "Invalid JSON output"

@handle_test_errors
@then("the output should validate against the schema")
def validate_output_step(context: Any) -> None:
    council_schema = file_handler.load_json_file("output.schema")
    assert file_handler.validate_json_schema(context.parse_result, council_schema), "Schema validation failed"



import json
import re
import requests
import sys
from tabulate import tabulate

def get_councils_from_files(branch):
    url = f"https://api.github.com/repos/robbrad/UKBinCollectionData/contents/uk_bin_collection/uk_bin_collection/councils?ref={branch}"
    response = requests.get(url)
    data = response.json()
    return [item['name'].replace('.py', '') for item in data if item['name'].endswith('.py')]

def get_councils_from_json(branch):
    url = f"https://raw.githubusercontent.com/robbrad/UKBinCollectionData/{branch}/uk_bin_collection/tests/input.json"
    response = requests.get(url)
    data = json.loads(response.text)
    return list(data.keys())

def get_councils_from_features(branch):
    url = f"https://raw.githubusercontent.com/robbrad/UKBinCollectionData/{branch}/uk_bin_collection/tests/features/validate_council_outputs.feature"
    response = requests.get(url)
    content = response.text
    return re.findall(r'Examples:\s+(\w+)', content)

def compare_councils(councils1, councils2, councils3):
    set1 = set(councils1)
    set2 = set(councils2)
    set3 = set(councils3)
    all_councils = set1 | set2 | set3
    all_council_data = {}
    discrepancies_found = False
    for council in all_councils:
        in_files = council in set1
        in_json = council in set2
        in_features = council in set3
        discrepancies_count = [in_files, in_json, in_features].count(False)
        all_council_data[council] = {
            'in_files': in_files,
            'in_json': in_json,
            'in_features': in_features,
            'discrepancies_count': discrepancies_count
        }
        if discrepancies_count > 0:
            discrepancies_found = True
    return all_council_data, discrepancies_found

def main(branch="master"):
    # Execute and print the comparison
    file_councils = get_councils_from_files(branch)
    json_councils = get_councils_from_json(branch)
    feature_councils = get_councils_from_features(branch)

    all_councils_data, discrepancies_found = compare_councils(file_councils, json_councils, feature_councils)

    # Create a list of lists for tabulate, sort by discrepancies count and then by name
    table_data = []
    headers = ["Council Name", "In Files", "In JSON", "In Features", "Discrepancies"]
    for council, presence in sorted(all_councils_data.items(), key=lambda x: (x[1]['discrepancies_count'], x[0])):
        row = [
            council,
            "✔" if presence['in_files'] else "✘",
            "✔" if presence['in_json'] else "✘",
            "✔" if presence['in_features'] else "✘",
            presence['discrepancies_count']
        ]
        table_data.append(row)

    # Print the table using tabulate
    print(tabulate(table_data, headers=headers, tablefmt='grid'))

    if discrepancies_found:
        print("Discrepancies found! Failing the workflow.")
        sys.exit(1)
    else:
        print("No discrepancies found. Workflow successful.")

if __name__ == "__main__":
    branch = sys.argv[1] if len(sys.argv) > 1 else "master"
    main(branch)



import argparse
import importlib
import os
import sys
import logging
from uk_bin_collection.uk_bin_collection.get_bin_data import (
    setup_logging,
    LOGGING_CONFIG,
)

_LOGGER = logging.getLogger(__name__)


# Dynamically importing the council processor
def import_council_module(module_name, src_path="councils"):
    module_path = os.path.realpath(os.path.join(os.path.dirname(__file__), src_path))
    if module_path not in sys.path:
        sys.path.append(module_path)
    return importlib.import_module(module_name)


class UKBinCollectionApp:
    def __init__(self):
        self.setup_arg_parser()
        self.parsed_args = None

    def setup_arg_parser(self):
        self.parser = argparse.ArgumentParser(
            description="UK Bin Collection Data Parser"
        )
        self.parser.add_argument(
            "module", type=str, help="Name of council module to use"
        )
        self.parser.add_argument(
            "URL", type=str, help="URL to parse - should be wrapped in double quotes"
        )
        self.parser.add_argument(
            "-p",
            "--postcode",
            type=str,
            help="Postcode to parse - should include a space and be wrapped in "
            "double-quotes",
            required=False,
        )
        self.parser.add_argument(
            "-n", "--number", type=str, help="House number to parse", required=False
        )
        self.parser.add_argument(
            "-s",
            "--skip_get_url",
            action="store_true",
            help="Skips the generic get_url - uses one in council class",
            required=False,
        )
        self.parser.add_argument(
            "-u", "--uprn", type=str, help="UPRN to parse", required=False
        )
        self.parser.add_argument(
            "-w",
            "--web_driver",
            help="URL for remote Selenium web driver - should be wrapped in double quotes",
            required=False,
        )
        self.parser.add_argument(
            "--headless",
            dest="headless",
            action="store_true",
            help="Should Selenium be headless. Defaults to true. Can be set to false to debug council",
        )
        self.parser.add_argument(
            "--not-headless",
            dest="headless",
            action="store_false",
            help="Should Selenium be headless. Defaults to true. Can be set to false to debug council",
        )
        self.parser.set_defaults(headless=True)

        self.parser.add_argument(
            "-d",
            "--dev_mode",
            action="store_true",
            help="Enables development mode - creates/updates entries in the input.json file for the council on each run",
            required=False,
        )
        self.parsed_args = None

    def set_args(self, args):
        self.parsed_args = self.parser.parse_args(args)

    def run(self):
        council_module_str = self.parsed_args.module
        council_module = import_council_module(council_module_str)
        address_url = self.parsed_args.URL
        postcode = self.parsed_args.postcode
        paon = self.parsed_args.number
        uprn = self.parsed_args.uprn
        skip_get_url = self.parsed_args.skip_get_url
        web_driver = self.parsed_args.web_driver
        headless = self.parsed_args.headless
        dev_mode = self.parsed_args.dev_mode

        return self.client_code(
            council_module.CouncilClass(),
            address_url,
            postcode=postcode,
            paon=paon,
            uprn=uprn,
            skip_get_url=skip_get_url,
            web_driver=web_driver,
            headless=headless,
            dev_mode=dev_mode,
            council_module_str=council_module_str,
        )

    def client_code(self, get_bin_data_class, address_url, **kwargs) -> None:
        """
        The client code calls the template method to execute the algorithm. Client
        code does not have to know the concrete class of an object it works with,
        as long as it works with objects through the interface of their base class.
        """
        return get_bin_data_class.template_method(address_url, **kwargs)


def run():
    global _LOGGER
    _LOGGER = setup_logging(LOGGING_CONFIG, None)
    app = UKBinCollectionApp()
    app.set_args(sys.argv[1:])
    print(app.run())


if __name__ == "__main__":
    run()
